# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/storage/v1alpha1/transfer_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.storage.v1alpha1 import transfer_pb2 as nebius_dot_storage_dot_v1alpha1_dot_transfer__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.nebius/storage/v1alpha1/transfer_service.proto\x12\x17nebius.storage.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a&nebius/storage/v1alpha1/transfer.proto\",\n\x12GetTransferRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"w\n\x14ListTransfersRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"x\n\x15ListTransfersResponse\x12\x37\n\x05items\x18\x01 \x03(\x0b\x32!.nebius.storage.v1alpha1.TransferR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\xa2\x01\n\x15\x43reateTransferRequest\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x41\n\x04spec\x18\x02 \x01(\x0b\x32%.nebius.storage.v1alpha1.TransferSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\"\xa2\x01\n\x15UpdateTransferRequest\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x41\n\x04spec\x18\x02 \x01(\x0b\x32%.nebius.storage.v1alpha1.TransferSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\"-\n\x13StopTransferRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"/\n\x15ResumeTransferRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"Z\n\x15\x44\x65leteTransferRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\x12)\n\x10resource_version\x18\x02 \x01(\x03R\x0fresourceVersion\"\x81\x01\n\x1aGetIterationHistoryRequest\x12\'\n\x0btransfer_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\ntransferId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"\x91\x01\n\x1bGetIterationHistoryResponse\x12J\n\niterations\x18\x01 \x03(\x0b\x32*.nebius.storage.v1alpha1.TransferIterationR\niterations\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xea\x06\n\x0fTransferService\x12U\n\x03Get\x12+.nebius.storage.v1alpha1.GetTransferRequest\x1a!.nebius.storage.v1alpha1.Transfer\x12R\n\tGetByName\x12\".nebius.common.v1.GetByNameRequest\x1a!.nebius.storage.v1alpha1.Transfer\x12\x65\n\x04List\x12-.nebius.storage.v1alpha1.ListTransfersRequest\x1a..nebius.storage.v1alpha1.ListTransfersResponse\x12U\n\x06\x43reate\x12..nebius.storage.v1alpha1.CreateTransferRequest\x1a\x1b.nebius.common.v1.Operation\x12U\n\x06Update\x12..nebius.storage.v1alpha1.UpdateTransferRequest\x1a\x1b.nebius.common.v1.Operation\x12Q\n\x04Stop\x12,.nebius.storage.v1alpha1.StopTransferRequest\x1a\x1b.nebius.common.v1.Operation\x12U\n\x06Resume\x12..nebius.storage.v1alpha1.ResumeTransferRequest\x1a\x1b.nebius.common.v1.Operation\x12U\n\x06\x44\x65lete\x12..nebius.storage.v1alpha1.DeleteTransferRequest\x1a\x1b.nebius.common.v1.Operation\x12\x80\x01\n\x13GetIterationHistory\x12\x33.nebius.storage.v1alpha1.GetIterationHistoryRequest\x1a\x34.nebius.storage.v1alpha1.GetIterationHistoryResponse\x1a\x13\xbaJ\x10transfer.storageBo\n\x1e\x61i.nebius.pub.storage.v1alpha1B\x14TransferServiceProtoP\x01Z5github.com/nebius/gosdk/proto/nebius/storage/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.storage.v1alpha1.transfer_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036ai.nebius.pub.storage.v1alpha1B\024TransferServiceProtoP\001Z5github.com/nebius/gosdk/proto/nebius/storage/v1alpha1'
  _GETTRANSFERREQUEST.fields_by_name['id']._options = None
  _GETTRANSFERREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _LISTTRANSFERSREQUEST.fields_by_name['parent_id']._options = None
  _LISTTRANSFERSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _CREATETRANSFERREQUEST.fields_by_name['metadata']._options = None
  _CREATETRANSFERREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _CREATETRANSFERREQUEST.fields_by_name['spec']._options = None
  _CREATETRANSFERREQUEST.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _UPDATETRANSFERREQUEST.fields_by_name['metadata']._options = None
  _UPDATETRANSFERREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _UPDATETRANSFERREQUEST.fields_by_name['spec']._options = None
  _UPDATETRANSFERREQUEST.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _STOPTRANSFERREQUEST.fields_by_name['id']._options = None
  _STOPTRANSFERREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _RESUMETRANSFERREQUEST.fields_by_name['id']._options = None
  _RESUMETRANSFERREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _DELETETRANSFERREQUEST.fields_by_name['id']._options = None
  _DELETETRANSFERREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _GETITERATIONHISTORYREQUEST.fields_by_name['transfer_id']._options = None
  _GETITERATIONHISTORYREQUEST.fields_by_name['transfer_id']._serialized_options = b'\272H\003\310\001\001'
  _TRANSFERSERVICE._options = None
  _TRANSFERSERVICE._serialized_options = b'\272J\020transfer.storage'
  _globals['_GETTRANSFERREQUEST']._serialized_start=237
  _globals['_GETTRANSFERREQUEST']._serialized_end=281
  _globals['_LISTTRANSFERSREQUEST']._serialized_start=283
  _globals['_LISTTRANSFERSREQUEST']._serialized_end=402
  _globals['_LISTTRANSFERSRESPONSE']._serialized_start=404
  _globals['_LISTTRANSFERSRESPONSE']._serialized_end=524
  _globals['_CREATETRANSFERREQUEST']._serialized_start=527
  _globals['_CREATETRANSFERREQUEST']._serialized_end=689
  _globals['_UPDATETRANSFERREQUEST']._serialized_start=692
  _globals['_UPDATETRANSFERREQUEST']._serialized_end=854
  _globals['_STOPTRANSFERREQUEST']._serialized_start=856
  _globals['_STOPTRANSFERREQUEST']._serialized_end=901
  _globals['_RESUMETRANSFERREQUEST']._serialized_start=903
  _globals['_RESUMETRANSFERREQUEST']._serialized_end=950
  _globals['_DELETETRANSFERREQUEST']._serialized_start=952
  _globals['_DELETETRANSFERREQUEST']._serialized_end=1042
  _globals['_GETITERATIONHISTORYREQUEST']._serialized_start=1045
  _globals['_GETITERATIONHISTORYREQUEST']._serialized_end=1174
  _globals['_GETITERATIONHISTORYRESPONSE']._serialized_start=1177
  _globals['_GETITERATIONHISTORYRESPONSE']._serialized_end=1322
  _globals['_TRANSFERSERVICE']._serialized_start=1325
  _globals['_TRANSFERSERVICE']._serialized_end=2199
# @@protoc_insertion_point(module_scope)



from enum import (
    IntEnum,
)
from typing import Any

import google.protobuf.descriptor as pb

from nebius.base.protos.descriptor import DescriptorWrap


class Enum(IntEnum):
    @classmethod
    def get_descriptor(cls) -> pb.EnumDescriptor:
        desc: Any = getattr(cls, "#descriptor", None)
        if desc is None:
            for val in cls.__dict__.values():
                if isinstance(val, DescriptorWrap):
                    desc = val()  # type: ignore[unused-ignore]
        if isinstance(desc, pb.EnumDescriptor):
            return desc
        raise ValueError(f"Descriptor not found in {cls.__name__}.")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v2/tenant_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.iam.v2 import tenant_pb2 as nebius_dot_iam_dot_v2_dot_tenant__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"nebius/iam/v2/tenant_service.proto\x12\rnebius.iam.v2\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a\x1anebius/iam/v2/tenant.proto\"\"\n\x10GetTenantRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"Q\n\x16GetTenantByNameRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"\xa5\x01\n\x13UpdateTenantRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12-\n\x04spec\x18\x02 \x01(\x0b\x32\x19.nebius.iam.v2.TenantSpecR\x04spec\x12\x1f\n\x0bname_prefix\x18\x03 \x01(\tR\nnamePrefix\"h\n\x12ListTenantsRequest\x12\x1b\n\tpage_size\x18\x01 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x02 \x01(\tR\tpageToken\x12\x16\n\x06\x66ilter\x18\x03 \x01(\tR\x06\x66ilter\"j\n\x13ListTenantsResponse\x12+\n\x05items\x18\x01 \x03(\x0b\x32\x15.nebius.iam.v2.TenantR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xbf\x02\n\rTenantService\x12=\n\x03Get\x12\x1f.nebius.iam.v2.GetTenantRequest\x1a\x15.nebius.iam.v2.Tenant\x12I\n\tGetByName\x12%.nebius.iam.v2.GetTenantByNameRequest\x1a\x15.nebius.iam.v2.Tenant\x12M\n\x04List\x12!.nebius.iam.v2.ListTenantsRequest\x1a\".nebius.iam.v2.ListTenantsResponse\x12I\n\x06Update\x12\".nebius.iam.v2.UpdateTenantRequest\x1a\x1b.nebius.common.v1.Operation\x1a\n\xbaJ\x07\x63pl.iamBY\n\x14\x61i.nebius.pub.iam.v2B\x12TenantServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v2.tenant_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v2B\022TenantServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v2'
  _GETTENANTBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETTENANTBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _TENANTSERVICE._options = None
  _TENANTSERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_GETTENANTREQUEST']._serialized_start=203
  _globals['_GETTENANTREQUEST']._serialized_end=237
  _globals['_GETTENANTBYNAMEREQUEST']._serialized_start=239
  _globals['_GETTENANTBYNAMEREQUEST']._serialized_end=320
  _globals['_UPDATETENANTREQUEST']._serialized_start=323
  _globals['_UPDATETENANTREQUEST']._serialized_end=488
  _globals['_LISTTENANTSREQUEST']._serialized_start=490
  _globals['_LISTTENANTSREQUEST']._serialized_end=594
  _globals['_LISTTENANTSRESPONSE']._serialized_start=596
  _globals['_LISTTENANTSRESPONSE']._serialized_end=702
  _globals['_TENANTSERVICE']._serialized_start=705
  _globals['_TENANTSERVICE']._serialized_end=1024
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Registry(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: RegistrySpec
    status: RegistryStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[RegistrySpec, _Mapping]] = ..., status: _Optional[_Union[RegistryStatus, _Mapping]] = ...) -> None: ...

class RegistrySpec(_message.Message):
    __slots__ = ["description", "images_count"]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    IMAGES_COUNT_FIELD_NUMBER: _ClassVar[int]
    description: str
    images_count: int
    def __init__(self, description: _Optional[str] = ..., images_count: _Optional[int] = ...) -> None: ...

class RegistryStatus(_message.Message):
    __slots__ = ["state", "images_count", "registry_fqdn"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        CREATING: _ClassVar[RegistryStatus.State]
        ACTIVE: _ClassVar[RegistryStatus.State]
        DELETING: _ClassVar[RegistryStatus.State]
        SUSPENDED: _ClassVar[RegistryStatus.State]
    CREATING: RegistryStatus.State
    ACTIVE: RegistryStatus.State
    DELETING: RegistryStatus.State
    SUSPENDED: RegistryStatus.State
    STATE_FIELD_NUMBER: _ClassVar[int]
    IMAGES_COUNT_FIELD_NUMBER: _ClassVar[int]
    REGISTRY_FQDN_FIELD_NUMBER: _ClassVar[int]
    state: RegistryStatus.State
    images_count: int
    registry_fqdn: str
    def __init__(self, state: _Optional[_Union[RegistryStatus.State, str]] = ..., images_count: _Optional[int] = ..., registry_fqdn: _Optional[str] = ...) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Backup(_message.Message):
    __slots__ = ["id", "source_cluster_id", "creation_start", "creation_finish", "source_cluster_name", "source_cluster_visible", "on_demand", "size_bytes"]
    ID_FIELD_NUMBER: _ClassVar[int]
    SOURCE_CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    CREATION_START_FIELD_NUMBER: _ClassVar[int]
    CREATION_FINISH_FIELD_NUMBER: _ClassVar[int]
    SOURCE_CLUSTER_NAME_FIELD_NUMBER: _ClassVar[int]
    SOURCE_CLUSTER_VISIBLE_FIELD_NUMBER: _ClassVar[int]
    ON_DEMAND_FIELD_NUMBER: _ClassVar[int]
    SIZE_BYTES_FIELD_NUMBER: _ClassVar[int]
    id: str
    source_cluster_id: str
    creation_start: _timestamp_pb2.Timestamp
    creation_finish: _timestamp_pb2.Timestamp
    source_cluster_name: str
    source_cluster_visible: bool
    on_demand: bool
    size_bytes: int
    def __init__(self, id: _Optional[str] = ..., source_cluster_id: _Optional[str] = ..., creation_start: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., creation_finish: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., source_cluster_name: _Optional[str] = ..., source_cluster_visible: bool = ..., on_demand: bool = ..., size_bytes: _Optional[int] = ...) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class NodeSetUnhealthyRequest(_message.Message):
    __slots__ = ["instance_id", "health_check_info", "dry_run"]
    class HealthCheckInfo(_message.Message):
        __slots__ = ["observed_at", "check_id", "description"]
        OBSERVED_AT_FIELD_NUMBER: _ClassVar[int]
        CHECK_ID_FIELD_NUMBER: _ClassVar[int]
        DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
        observed_at: _timestamp_pb2.Timestamp
        check_id: str
        description: str
        def __init__(self, observed_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., check_id: _Optional[str] = ..., description: _Optional[str] = ...) -> None: ...
    INSTANCE_ID_FIELD_NUMBER: _ClassVar[int]
    HEALTH_CHECK_INFO_FIELD_NUMBER: _ClassVar[int]
    DRY_RUN_FIELD_NUMBER: _ClassVar[int]
    instance_id: str
    health_check_info: NodeSetUnhealthyRequest.HealthCheckInfo
    dry_run: bool
    def __init__(self, instance_id: _Optional[str] = ..., health_check_info: _Optional[_Union[NodeSetUnhealthyRequest.HealthCheckInfo, _Mapping]] = ..., dry_run: bool = ...) -> None: ...

class NodeSetUnhealthyResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/postgresql/v1alpha1/config/postgresql.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6nebius/msp/postgresql/v1alpha1/config/postgresql.proto\x12%nebius.msp.postgresql.v1alpha1.config\x1a\x1b\x62uf/validate/validate.proto\"\xf6\t\n\x12PostgresqlConfig16\x12K\n\x13\x61utovacuum_work_mem\x18\x01 \x01(\x03\x42\x16\xbaH\x13\"\x11\x18\xff\xff\xff\xff\x07(\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01H\x00R\x11\x61utovacuumWorkMem\x88\x01\x01\x12?\n\x11statement_timeout\x18\x02 \x01(\x03\x42\r\xbaH\n\"\x08\x18\xff\xff\xff\xff\x07(\x00H\x01R\x10statementTimeout\x88\x01\x01\x12`\n#idle_in_transaction_session_timeout\x18\x03 \x01(\x03\x42\r\xbaH\n\"\x08\x18\xff\xff\xff\xff\x07(\x00H\x02R\x1fidleInTransactionSessionTimeout\x88\x01\x01\x12X\n\x1c\x61utovacuum_vacuum_cost_delay\x18\x04 \x01(\x03\x42\x12\xbaH\x0f\"\r\x18\x64(\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01H\x03R\x19\x61utovacuumVacuumCostDelay\x88\x01\x01\x12Y\n\x1c\x61utovacuum_vacuum_cost_limit\x18\x05 \x01(\x03\x42\x13\xbaH\x10\"\x0e\x18\x90N(\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01H\x04R\x19\x61utovacuumVacuumCostLimit\x88\x01\x01\x12@\n\x12\x61utovacuum_naptime\x18\x06 \x01(\x03\x42\x0c\xbaH\t\"\x07\x18\x9b\x89\x83\x01(\x01H\x05R\x11\x61utovacuumNaptime\x88\x01\x01\x12\x61\n\x1e\x61utovacuum_vacuum_scale_factor\x18\x07 \x01(\x01\x42\x17\xbaH\x14\x12\x12\x19\x00\x00\x00\x00\x00\x00Y@)\x00\x00\x00\x00\x00\x00\x00\x00H\x06R\x1b\x61utovacuumVacuumScaleFactor\x88\x01\x01\x12\x63\n\x1f\x61utovacuum_analyze_scale_factor\x18\x08 \x01(\x01\x42\x17\xbaH\x14\x12\x12\x19\x00\x00\x00\x00\x00\x00Y@)\x00\x00\x00\x00\x00\x00\x00\x00H\x07R\x1c\x61utovacuumAnalyzeScaleFactor\x88\x01\x01\x12\x46\n\x1d\x64\x65\x66\x61ult_transaction_read_only\x18\t \x01(\x08H\x08R\x1a\x64\x65\x66\x61ultTransactionReadOnly\x88\x01\x01\x12$\n\x0bsearch_path\x18\n \x01(\tH\tR\nsearchPath\x88\x01\x01\x12\x38\n\x0fmax_connections\x18\x0b \x01(\x03\x42\n\xbaH\x07\"\x05\x18\x80\x64(dH\nR\x0emaxConnections\x88\x01\x01\x12:\n\x0eshared_buffers\x18\x0c \x01(\x03\x42\x0e\xbaH\x0b\"\t\x18\xf8\xff\xff\xff\x1f(\x80\x01H\x0bR\rsharedBuffers\x88\x01\x01\x42\x16\n\x14_autovacuum_work_memB\x14\n\x12_statement_timeoutB&\n$_idle_in_transaction_session_timeoutB\x1f\n\x1d_autovacuum_vacuum_cost_delayB\x1f\n\x1d_autovacuum_vacuum_cost_limitB\x15\n\x13_autovacuum_naptimeB!\n\x1f_autovacuum_vacuum_scale_factorB\"\n _autovacuum_analyze_scale_factorB \n\x1e_default_transaction_read_onlyB\x0e\n\x0c_search_pathB\x12\n\x10_max_connectionsB\x11\n\x0f_shared_buffersB\x86\x01\n,ai.nebius.pub.msp.postgresql.v1alpha1.configB\x0fPostgresqlProtoP\x01ZCgithub.com/nebius/gosdk/proto/nebius/msp/postgresql/v1alpha1/configb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.postgresql.v1alpha1.config.postgresql_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n,ai.nebius.pub.msp.postgresql.v1alpha1.configB\017PostgresqlProtoP\001ZCgithub.com/nebius/gosdk/proto/nebius/msp/postgresql/v1alpha1/config'
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_work_mem']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_work_mem']._serialized_options = b'\272H\023\"\021\030\377\377\377\377\007(\377\377\377\377\377\377\377\377\377\001'
  _POSTGRESQLCONFIG16.fields_by_name['statement_timeout']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['statement_timeout']._serialized_options = b'\272H\n\"\010\030\377\377\377\377\007(\000'
  _POSTGRESQLCONFIG16.fields_by_name['idle_in_transaction_session_timeout']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['idle_in_transaction_session_timeout']._serialized_options = b'\272H\n\"\010\030\377\377\377\377\007(\000'
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_vacuum_cost_delay']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_vacuum_cost_delay']._serialized_options = b'\272H\017\"\r\030d(\377\377\377\377\377\377\377\377\377\001'
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_vacuum_cost_limit']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_vacuum_cost_limit']._serialized_options = b'\272H\020\"\016\030\220N(\377\377\377\377\377\377\377\377\377\001'
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_naptime']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_naptime']._serialized_options = b'\272H\t\"\007\030\233\211\203\001(\001'
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_vacuum_scale_factor']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_vacuum_scale_factor']._serialized_options = b'\272H\024\022\022\031\000\000\000\000\000\000Y@)\000\000\000\000\000\000\000\000'
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_analyze_scale_factor']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['autovacuum_analyze_scale_factor']._serialized_options = b'\272H\024\022\022\031\000\000\000\000\000\000Y@)\000\000\000\000\000\000\000\000'
  _POSTGRESQLCONFIG16.fields_by_name['max_connections']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['max_connections']._serialized_options = b'\272H\007\"\005\030\200d(d'
  _POSTGRESQLCONFIG16.fields_by_name['shared_buffers']._options = None
  _POSTGRESQLCONFIG16.fields_by_name['shared_buffers']._serialized_options = b'\272H\013\"\t\030\370\377\377\377\037(\200\001'
  _globals['_POSTGRESQLCONFIG16']._serialized_start=127
  _globals['_POSTGRESQLCONFIG16']._serialized_end=1397
# @@protoc_insertion_point(module_scope)



from .descriptors import Message

_cache = dict[str, bool]()


def is_recursive(msg: Message) -> bool:
    """Returns true only if the message has a field of any level with itself. Messages
    containing recursive ones won't themselves be reported as recursive.

    Args:
        msg (Message): message to test

    Returns:
        bool: if the message is recursive
    """
    if msg.full_type_name in _cache:
        return _cache[msg.full_type_name]
    return _is_recursive_detector(msg, set[str]())


def _is_recursive_detector(current: Message, visited: set[str]) -> bool:
    if current.full_type_name in _cache:
        return _cache[current.full_type_name]
    if current.full_type_name in visited:
        _cache[current.full_type_name] = True
        return True

    visited.add(current.full_type_name)
    for field in current.fields():
        if field.is_map() and field.map_value.is_message():
            field = field.map_value
        if field.is_message():
            _is_recursive_detector(field.message, visited)
    visited.remove(current.full_type_name)
    if current.full_type_name not in _cache:
        _cache[current.full_type_name] = False
    return _cache[current.full_type_name]



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius.common.v1 import operation_pb2 as _operation_pb2
from nebius.api.nebius.mk8s.v1 import node_group_pb2 as _node_group_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GetNodeGroupCompatibilityMatrixRequest(_message.Message):
    __slots__ = ["cluster_kubernetes_version", "platform"]
    CLUSTER_KUBERNETES_VERSION_FIELD_NUMBER: _ClassVar[int]
    PLATFORM_FIELD_NUMBER: _ClassVar[int]
    cluster_kubernetes_version: str
    platform: str
    def __init__(self, cluster_kubernetes_version: _Optional[str] = ..., platform: _Optional[str] = ...) -> None: ...

class NodeGroupCompatibilityMatrix(_message.Message):
    __slots__ = ["versions"]
    VERSIONS_FIELD_NUMBER: _ClassVar[int]
    versions: _containers.RepeatedCompositeFieldContainer[NodeGroupCompatibilityVersion]
    def __init__(self, versions: _Optional[_Iterable[_Union[NodeGroupCompatibilityVersion, _Mapping]]] = ...) -> None: ...

class NodeGroupCompatibilityVersion(_message.Message):
    __slots__ = ["kubernetes_version", "items"]
    KUBERNETES_VERSION_FIELD_NUMBER: _ClassVar[int]
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    kubernetes_version: str
    items: _containers.RepeatedCompositeFieldContainer[NodeGroupCompatibilityVersionItem]
    def __init__(self, kubernetes_version: _Optional[str] = ..., items: _Optional[_Iterable[_Union[NodeGroupCompatibilityVersionItem, _Mapping]]] = ...) -> None: ...

class NodeGroupCompatibilityVersionItem(_message.Message):
    __slots__ = ["os", "drivers_preset", "compatible_platforms"]
    OS_FIELD_NUMBER: _ClassVar[int]
    DRIVERS_PRESET_FIELD_NUMBER: _ClassVar[int]
    COMPATIBLE_PLATFORMS_FIELD_NUMBER: _ClassVar[int]
    os: str
    drivers_preset: str
    compatible_platforms: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, os: _Optional[str] = ..., drivers_preset: _Optional[str] = ..., compatible_platforms: _Optional[_Iterable[str]] = ...) -> None: ...

class CreateNodeGroupRequest(_message.Message):
    __slots__ = ["metadata", "spec"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: _node_group_pb2.NodeGroupSpec
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[_node_group_pb2.NodeGroupSpec, _Mapping]] = ...) -> None: ...

class GetNodeGroupRequest(_message.Message):
    __slots__ = ["id", "resource_version"]
    ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_VERSION_FIELD_NUMBER: _ClassVar[int]
    id: str
    resource_version: str
    def __init__(self, id: _Optional[str] = ..., resource_version: _Optional[str] = ...) -> None: ...

class ListNodeGroupsRequest(_message.Message):
    __slots__ = ["parent_id", "page_size", "page_token"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    page_size: int
    page_token: str
    def __init__(self, parent_id: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ...) -> None: ...

class ListNodeGroupsResponse(_message.Message):
    __slots__ = ["items", "next_page_token"]
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[_node_group_pb2.NodeGroup]
    next_page_token: str
    def __init__(self, items: _Optional[_Iterable[_Union[_node_group_pb2.NodeGroup, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...

class UpdateNodeGroupRequest(_message.Message):
    __slots__ = ["metadata", "spec"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: _node_group_pb2.NodeGroupSpec
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[_node_group_pb2.NodeGroupSpec, _Mapping]] = ...) -> None: ...

class DeleteNodeGroupRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class UpgradeNodeGroupRequest(_message.Message):
    __slots__ = ["id", "latest_infra_version"]
    ID_FIELD_NUMBER: _ClassVar[int]
    LATEST_INFRA_VERSION_FIELD_NUMBER: _ClassVar[int]
    id: str
    latest_infra_version: _empty_pb2.Empty
    def __init__(self, id: _Optional[str] = ..., latest_infra_version: _Optional[_Union[_empty_pb2.Empty, _Mapping]] = ...) -> None: ...



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QuotaAllowanceSpec(_message.Message):
    __slots__ = ["limit", "region"]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    REGION_FIELD_NUMBER: _ClassVar[int]
    limit: int
    region: str
    def __init__(self, limit: _Optional[int] = ..., region: _Optional[str] = ...) -> None: ...

class QuotaAllowanceStatus(_message.Message):
    __slots__ = ["state", "usage", "service", "description", "service_description", "unit", "usage_percentage", "usage_state"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[QuotaAllowanceStatus.State]
        STATE_PROVISIONING: _ClassVar[QuotaAllowanceStatus.State]
        STATE_ACTIVE: _ClassVar[QuotaAllowanceStatus.State]
        STATE_FROZEN: _ClassVar[QuotaAllowanceStatus.State]
        STATE_DELETED: _ClassVar[QuotaAllowanceStatus.State]
    STATE_UNSPECIFIED: QuotaAllowanceStatus.State
    STATE_PROVISIONING: QuotaAllowanceStatus.State
    STATE_ACTIVE: QuotaAllowanceStatus.State
    STATE_FROZEN: QuotaAllowanceStatus.State
    STATE_DELETED: QuotaAllowanceStatus.State
    class UsageState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        USAGE_STATE_UNSPECIFIED: _ClassVar[QuotaAllowanceStatus.UsageState]
        USAGE_STATE_USED: _ClassVar[QuotaAllowanceStatus.UsageState]
        USAGE_STATE_NOT_USED: _ClassVar[QuotaAllowanceStatus.UsageState]
        USAGE_STATE_UNKNOWN: _ClassVar[QuotaAllowanceStatus.UsageState]
        USAGE_STATE_NOT_APPLICABLE: _ClassVar[QuotaAllowanceStatus.UsageState]
    USAGE_STATE_UNSPECIFIED: QuotaAllowanceStatus.UsageState
    USAGE_STATE_USED: QuotaAllowanceStatus.UsageState
    USAGE_STATE_NOT_USED: QuotaAllowanceStatus.UsageState
    USAGE_STATE_UNKNOWN: QuotaAllowanceStatus.UsageState
    USAGE_STATE_NOT_APPLICABLE: QuotaAllowanceStatus.UsageState
    STATE_FIELD_NUMBER: _ClassVar[int]
    USAGE_FIELD_NUMBER: _ClassVar[int]
    SERVICE_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    SERVICE_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    UNIT_FIELD_NUMBER: _ClassVar[int]
    USAGE_PERCENTAGE_FIELD_NUMBER: _ClassVar[int]
    USAGE_STATE_FIELD_NUMBER: _ClassVar[int]
    state: QuotaAllowanceStatus.State
    usage: int
    service: str
    description: str
    service_description: str
    unit: str
    usage_percentage: str
    usage_state: QuotaAllowanceStatus.UsageState
    def __init__(self, state: _Optional[_Union[QuotaAllowanceStatus.State, str]] = ..., usage: _Optional[int] = ..., service: _Optional[str] = ..., description: _Optional[str] = ..., service_description: _Optional[str] = ..., unit: _Optional[str] = ..., usage_percentage: _Optional[str] = ..., usage_state: _Optional[_Union[QuotaAllowanceStatus.UsageState, str]] = ...) -> None: ...

class QuotaAllowance(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: QuotaAllowanceSpec
    status: QuotaAllowanceStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[QuotaAllowanceSpec, _Mapping]] = ..., status: _Optional[_Union[QuotaAllowanceStatus, _Mapping]] = ...) -> None: ...



"""Request option keys used by the token renewal machinery.

This module defines string constants that are used as keys in the
``options`` mapping passed to per-request :class:`Receiver.fetch` calls
and similar APIs. The values associated with these keys are interpreted
by the renewal and exchangeable bearers to control retry and renewal
behaviour.

General conventions
-------------------
- Boolean flags: options that act as switches (for example
  :data:`OPTION_RENEW_SYNCHRONOUS`) are enabled when the key is present
  in the options mapping and its value is not an empty string. The exact
  string value is ignored; presence/non-empty is considered ``True``.
- Numeric overrides: some options accept numeric values encoded as
  strings (for example :data:`OPTION_MAX_RETRIES` and
  :data:`OPTION_RENEW_REQUEST_TIMEOUT`). These are parsed using
  :func:`int` or :func:`float` respectively; invalid values are logged
  and ignored, falling back to implementation defaults.

Options
-------

:data:`OPTION_MAX_RETRIES`
        Key: ``"max_fetch_token_retries"``

        When present in the request ``options`` mapping this key overrides
        the configured maximum number of fetch retries for the current
        receiver. The value must be a decimal integer encoded as a string
        (for example ``"3"``). If the value cannot be parsed as an
        integer it is ignored and the receiver's configured default is
        used (typically ``2``).

:data:`OPTION_RENEW_REQUIRED`
        Key: ``"token_renew_required"``

        A boolean-like flag that forces the bearer to perform a renewal
        even if the cached token would normally be considered fresh. Any
        non-empty string value will enable the behaviour; typically the key
        is set to ``"1"`` or ``"true"`` in examples.

:data:`OPTION_RENEW_SYNCHRONOUS`
        Key: ``"token_renew_synchronous"``

        When present and non-empty this flag requests a synchronous renewal
        for the current fetch operation. In synchronous mode the caller
        waits for a fresh token to be retrieved before :meth:`Receiver.fetch`
        returns (subject to the usual timeout). Note that synchronous
        requests may also suppress triggering a background renewal from the
        receiver's retry logic.

:data:`OPTION_REPORT_ERROR`
        Key: ``"token_renew_report_error"``

        When set, renewal errors are reported back to the caller of
        :meth:`Receiver.fetch` by raising an exception instead of being
        handled silently by background retry logic. This is useful when the
        client needs immediate visibility of renewal failures.

:data:`OPTION_RENEW_REQUEST_TIMEOUT`
        Key: ``"token_renew_request_timeout"``

        Numeric override controlling the timeout (in seconds) used for a
        synchronous renewal request. The value must be a floating-point
        number encoded as a string (for example ``"2.5"``). If the value
        cannot be parsed as a float the option is ignored and the bearer's
        configured default timeout is used (typically 5 seconds).

Examples
--------

Typical usage when requesting a synchronous renewal with a short
timeout::

        options = {
                OPTION_RENEW_REQUIRED: "1",
                OPTION_RENEW_SYNCHRONOUS: "1",
                OPTION_RENEW_REQUEST_TIMEOUT: "0.2",
        }

Override the maximum number of retries for a specific fetch::

        options = {
                OPTION_MAX_RETRIES: "5",
        }

Notes
-----
Implementations that read these options generally follow the patterns
used in :mod:`nebius.aio.token.renewable` and
:mod:`nebius.aio.token.exchangeable`: boolean flags are tested by
checking for a non-empty value, while numeric overrides are parsed and
validated. Invalid numeric values are logged and ignored.
"""

OPTION_MAX_RETRIES = "max_fetch_token_retries"
"""Override the maximum number of token fetch retries for a
single receiver.

When provided in a request ``options`` mapping the value must be a
decimal integer encoded as a string (for example ``"3"``). If the
value cannot be parsed as an integer the option is ignored and the
receiver's configured default is used.
"""
OPTION_RENEW_REQUIRED = "token_renew_required"
"""Boolean-like flag that forces a token renewal.

Any non-empty string value (commonly ``"1"`` or ``"true"``)
enables the behaviour; presence of the key signals that the cached
token should be refreshed even if it would normally be considered
fresh.
"""
OPTION_RENEW_SYNCHRONOUS = "token_renew_synchronous"
"""Request a synchronous renewal for the current fetch.

When this key is present with a non-empty value the fetch operation
will block until a new token has been retrieved (subject to the
fetch timeout and the optional :data:`OPTION_RENEW_REQUEST_TIMEOUT`
override).
"""
OPTION_REPORT_ERROR = "token_renew_report_error"
"""When set, report renewal errors to the caller.

If present and non-empty renewal errors are propagated to the caller
of :meth:`Receiver.fetch` (instead of being handled by background
retry logic). This is useful for callers that need immediate
visibility of failures.
"""
OPTION_RENEW_REQUEST_TIMEOUT = "token_renew_request_timeout"
"""Floating-point timeout (in seconds) used for synchronous
renewal requests.

The value is parsed using :func:`float` (for example ``"2.5"``).
Invalid values are ignored and the bearer's configured default is
used.
"""



from time import time

from bs4 import BeautifulSoup, CData, NavigableString, PageElement
from bs4 import Comment as BSComment
from bs4 import Tag as BSTag
from pydoctor.model import System  # type: ignore
from pydoctor.templatewriter import TemplateLookup  # type: ignore
from pydoctor.templatewriter import (  # type: ignore[unused-ignore]
    TemplateWriter as Base,
)
from pydoctor.templatewriter.pages import Page  # type: ignore
from pydoctor.templatewriter.writer import flattenToFile  # type: ignore
from twisted.web.template import CDATA, Comment, Tag, renderer


def bs4_to_tag(element: PageElement) -> Tag | CDATA | Comment | str:
    if isinstance(element, BSTag):
        tag = Tag(element.name)
        for k, v in element.attrs.items():
            tag.attributes[k] = v
        for child in element.contents:
            tag.children.append(bs4_to_tag(child))
        return tag
    if isinstance(element, CData):
        return CDATA(element.text)
    if isinstance(element, BSComment):
        return Comment(element.text)
    if isinstance(element, NavigableString):
        return element.text
    raise ValueError(f"Unsupported element type: {type(element)}")


class APIReferencePage(Page):  # type:ignore[misc]
    filename = "apiReference.html"

    def __init__(
        self,
        system: System,
        template_lookup: TemplateLookup,
        clients: BSTag | None,
    ) -> None:
        super().__init__(system=system, template_lookup=template_lookup)
        self._clients = clients

    def title(self) -> str:
        return "Nebius API Reference"

    @renderer
    def clients(self, request: object, tag: Tag) -> Tag:
        if self._clients is not None:
            tag(bs4_to_tag(self._clients))
        return tag


CLIENT_SELECTOR = 'li a[name="nebius.aio.client.Client"] ~ ul'


class TemplateWriter(Base):  # type:ignore[misc]
    def writeSummaryPages(  # noqa: N802 # comply with the protocol
        self, system: System
    ) -> None:
        super().writeSummaryPages(system)

        system.msg("html", "starting " + APIReferencePage.filename + " ...", nonl=True)
        start = time()
        fn = self.build_directory.joinpath("classIndex.html")
        with fn.open("r", encoding="utf-8") as fp:
            soup = BeautifulSoup(fp, "html.parser")
        clients = soup.select_one(CLIENT_SELECTOR)
        page = APIReferencePage(system, self.template_lookup, clients)
        fn = self.build_directory.joinpath(page.filename)
        with fn.open("wb") as fobj:
            flattenToFile(fobj, page)  # type: ignore[unused-ignore]
        system.msg("html", "took %fs" % (time() - start), wantsnl=False)
